"""Freesound client"""
import json
import webbrowser
import os

import requests
import freesound

from termcolor import colored


class Client:
    """Freesound client"""

    FS_CLIENT_ID = "vPkOlpykBDA4fU9fzGmE"
    token_data = {}
    client = None
    path = ""

    def __init__(self, path):
        self.path = path
        os.makedirs(self.path, exist_ok=True)
        try:
            with open(
                os.path.join(self.path, "token_data"), "r", encoding="UTF-8"
            ) as file:
                self.token_data = json.load(file)
        except IOError:
            self.token_data = {}
        if "access_token" not in self.token_data:
            self._authorize()

        self.client = freesound.FreesoundClient()
        self.client.set_token(self.token_data["access_token"], "oauth")

        self._refresh_token()

    def _authorize(self):
        url = (
            "https://freesound.org/apiv2/oauth2/authorize/?client_id="
            + self.FS_CLIENT_ID
            + "&response_type=code"
        )
        print("")
        input(
            colored(
                "OAuth2 token needed. Press Enter to continue and come back with a code...",
                "blue",
            )
        )
        print("")
        print("If a browser has not opened, please manually go to " + url)
        print("")
        webbrowser.open(url)
        code = input("Please enter authorization code: ")
        url = "https://freesound.org/apiv2/oauth2/access_token/"
        params = {
            "client_id": self.FS_CLIENT_ID,
            "grant_type": "authorization_code",
            "code": code,
        }
        response = requests.post(url, params, timeout=5)
        if response.status_code == 200:
            self.token_data = response.json()
            with open(
                os.path.join(self.path, "token_data"), "w", encoding="UTF-8"
            ) as file:
                json.dump(self.token_data, file)
        else:
            raise Exception("An error occured while getting access token.", response)

    def _refresh_token(self):
        # Trying to get current user, to check if OAuth2 token is correct.
        uri = freesound.URIS.uri(freesound.URIS.ME)
        try:
            user = freesound.FSRequest.request(uri, None, self.client, freesound.User)
            print("Logged in as " + user.username)
        except freesound.FreesoundException as exception:
            if exception.code == 401:
                url = "https://freesound.org/apiv2/oauth2/access_token/"
                params = {
                    "client_id": self.FS_CLIENT_ID,
                    "grant_type": "refresh_token",
                    "refresh_token": self.token_data["refresh_token"],
                }
                response = requests.post(url, params, timeout=5)
                if response.status_code == 200:
                    self.token_data = response.json()
                    with open(
                        os.path.join(self.path, "token_data"), "w", encoding="UTF-8"
                    ) as file:
                        json.dump(self.token_data, file)
                    self.client.set_token(self.token_data["access_token"], "oauth")
                else:
                    raise Exception(
                        "An error occured while refreshing access token.", response
                    ) from exception
            else:
                raise exception

    def __getattr__(self, attr):
        def wrapped_method(*args, **kwargs):
            # try:
            result = getattr(self.client, attr)(*args, **kwargs)
            # except freesound.FreesoundException as exception:
            #     if exception.code == 401:
            #         self._refresh_token()
            #         result = getattr(self.client, attr)(*args, **kwargs)
            #     else:
            #         raise exception
            return result

        return wrapped_method



#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""Display utilities"""

from termcolor import colored


def print_error(message, exception=None):
    """Print a frigging error"""
    print("")
    print(colored(message, "red"))
    print("")
    if exception:
        print(exception)



from flask import Flask
from shabda.dj import Dj


def create_app():
    app = Flask(__name__)

    from . import web

    app.register_blueprint(web.bp)

    return app


if __name__ == "__main__":
    print("Hello world")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""Shabda engine"""

from functools import partial
import random
import os
import asyncio
import urllib

import freesound
import pydub
from termcolor import colored
from google.cloud import texttospeech

from shabda.display import print_error
from shabda.client import Client
from shabda.sampleset import FREESOUND, SampleSet, TTS
from shabda.sound import Sound


class Dj:
    """Shabda engine"""

    client = None
    samples_path = ""

    def __init__(self, config_path="", samples_path=""):
        self.client = Client(config_path)
        self.samples_path = samples_path

    def parse_definition(self, definition):
        """Parse a pack definition"""
        words = {}
        sections = definition.split(",")
        for section in sections:
            parts = section.split(":")
            rawword = parts[0]
            word = "".join(ch for ch in rawword if ch.isalnum() or ch == "_")
            if len(word) == 0:
                raise ValueError("A sample name is required")
            number = None
            if len(parts) > 1:
                try:
                    number = int(parts[1])
                except ValueError as exception:
                    raise ValueError(
                        "A valid sample number is required after the colon"
                    ) from exception
                if number < 1:
                    raise ValueError("A sample number must be greater than 0")
                if number > 10:
                    raise ValueError("A sample number must be less than 11")
            words[word] = number
        return words

    def list(
        self,
        word,
        max_number=None,
        licenses=None,
        gender=None,
        language=None,
        soundtype=None,
    ):
        """List files for a sample name"""
        if soundtype == "tts":
            stype = TTS
        else:
            stype = FREESOUND
        sampleset = SampleSet(word, stype, self.samples_path)
        return sampleset.list(
            max_number, licenses=licenses, gender=gender, language=language
        )

    async def speak(self, word, language, gender):
        """Speak a word"""
        sampleset = SampleSet(word, TTS, self.samples_path)
        existing_samples = sampleset.list()
        if len(existing_samples) > 0:
            return True
        word_dir = sampleset.dir()
        client = texttospeech.TextToSpeechClient()
        synthesis_input = texttospeech.SynthesisInput(text=word.replace("_", " "))
        # mini hack
        if language == "en-GB" and gender == "f":
            voice = texttospeech.VoiceSelectionParams(
                name="en-GB-Neural2-A",
                language_code="en-GB",
                ssml_gender=texttospeech.SsmlVoiceGender.FEMALE,
            )
            # speaking_rate=0.85
            # pitch=-4
        else:
            if gender == "m":
                ssml_gender = texttospeech.SsmlVoiceGender.MALE
            else:
                ssml_gender = texttospeech.SsmlVoiceGender.FEMALE
            voice = texttospeech.VoiceSelectionParams(
                language_code=language,
                ssml_gender=ssml_gender,
            )
        audio_config = texttospeech.AudioConfig(
            audio_encoding=texttospeech.AudioEncoding.LINEAR16,
            # speaking_rate=0.85,
            # pitch=-4,
        )
        response = client.synthesize_speech(
            input=synthesis_input, voice=voice, audio_config=audio_config
        )
        filepath = word_dir + "/" + word + "_0.wav"
        with open(filepath, "wb") as out:
            out.write(response.audio_content)
        sound = Sound(
            speechsound={
                "gender": gender,
                "language": language,
                "file": filepath,
            }
        )
        sampleset.add(sound)
        sampleset.saveconfig()
        return True

    async def fetch(self, word, num, licenses):
        """Fetch a collection of samples"""
        mastersound = None
        sampleset = SampleSet(word, samples_path=self.samples_path)

        existing_samples = sampleset.list(num, licenses=licenses)
        existing_number = len(existing_samples)
        if existing_number >= num:
            return True

        master_id = sampleset.master_id
        if master_id:
            mastersound = self.client.get_sound(
                master_id, fields="id,name,type,duration,previews"
            )
            print(colored("Master sound exists...", "green"))

        if not mastersound:
            mastersound = await self.search_master_sound(word)

        if not mastersound:
            sampleset.clean()
            return False

        sampleset.master_id = mastersound.id

        print("Sound found: " + mastersound.name)

        similar = mastersound.get_similar(
            fields="id,name,type,duration,previews,license,username,url",
            page_size=100,
        )

        print("Found " + str(len(similar.results)) + " similar sounds.")

        ssounds = []
        for result in similar:
            ssound = Sound(freesound=result)
            if existing_number >= num:
                break
            if result.id == mastersound.id:
                continue
            if result.duration > 5:
                continue
            if sampleset.contains(result.id):
                continue
            if len(licenses) and not ssound.licensename in licenses:
                continue
            ssounds.append(result)
            existing_number += 1

        print("Remaining " + str(len(ssounds)) + " similar sounds after filtering.")

        sample_num = len(existing_samples)
        tasks = []
        for ssound in ssounds:
            tasks.append(self.download(sampleset, ssound, sample_num))
            sample_num += 1
        await asyncio.gather(*tasks)

        sampleset.saveconfig()

        if len(sampleset.list()) == 0:
            sampleset.clean()
            return False

        return True

    def random_word(self):
        """Choose a random word"""
        file = os.path.dirname(__file__) + "/" + "1000nouns.txt"
        words = open(file, "r", encoding="utf-8").read().splitlines()
        word = random.choice(words)
        return word

    async def search_master_sound(self, word):
        """Search master sound for a word"""
        found = None
        while found is None:
            print("")
            print(colored('Searching freesound for "' + word + '"...', "green"))
            print("")
            loop = asyncio.get_event_loop()
            results = await loop.run_in_executor(
                None,
                partial(
                    self.client.text_search,
                    query=word,
                    fields="id,name,type,duration,previews,license,username,url",
                    page_size=10,
                    filter='duration:[* TO 1] license:"Creative Commons 0"',
                ),
            )

            if len(results.results) == 0:
                print_error("No found samples.")
                word = self.random_word()
            else:
                found = True

        key = random.randint(0, len(results.results) - 1)
        sound = results[key]
        return sound

    def trim(self, sound):
        """Trim silence"""

        # We can modify the silence_threshold of the detect_leading_silence function:
        # detect_leading_silence(sound, silence_threshold=-50.0, chunk_size=10)
        # in order to cut more or less

        def trim_leading_silence(segment: pydub.AudioSegment) -> pydub.AudioSegment:
            return segment[pydub.silence.detect_leading_silence(segment) :]

        def trim_trailing_silence(segment: pydub.AudioSegment) -> pydub.AudioSegment:
            return trim_leading_silence(segment.reverse()).reverse()

        return trim_trailing_silence(trim_leading_silence(sound))

    async def download(self, sampleset: SampleSet, ssound, sample_num):
        """Download a sample, normalize and cut it"""

        def match_target_amplitude(sound, target_dbfs):
            change_in_dbfs = target_dbfs - sound.dBFS
            return sound.apply_gain(change_in_dbfs)

        word_dir = sampleset.dir()
        try:
            source_name = str(sample_num) + "-source"
            source_path = word_dir + "/" + source_name

            print("Dowloading " + word_dir + " sample #" + str(sample_num) + "...")
            loop = asyncio.get_event_loop()
            await loop.run_in_executor(None, ssound.retrieve, word_dir, source_name)

            sound = pydub.AudioSegment.from_file(source_path)
            sound = sound.set_frame_rate(44100)
            sound = sound.set_channels(1)
            sound = sound.set_sample_width(2)
            sound = match_target_amplitude(sound, -20.0)

            sound = self.trim(sound)
            export_num = sample_num
            while True:
                export_name = sampleset.word + "_" + str(export_num) + ".wav"
                export_path = word_dir + "/" + export_name
                if not os.path.exists(export_path):
                    break
                export_num += 1
            sound.export(export_path, format="wav")

            shabdasound = Sound(freesound=ssound)
            shabdasound.file = export_path
            sampleset.add(shabdasound)
            print("Sample " + word_dir + "#" + str(sample_num) + " downloaded!")
        except pydub.exceptions.CouldntDecodeError as exception:
            print_error("Error while decoding source file", exception)
        except freesound.FreesoundException as exception:
            print_error("Error while downloading sound", exception)
        except urllib.error.ContentTooShortError as exception:
            print_error("Content to short", exception)
        if os.path.exists(source_path):
            os.remove(source_path)



"""A single sound file and mappings from Freesound"""


class Sound:
    """A sound file"""

    id = None
    username = None
    url = None
    licensename = None
    file = None
    language = None
    gender = None

    def __init__(self, freesound=None, configsound=None, speechsound=None):
        if freesound is not None:
            self.id = freesound.id
            self.username = freesound.username
            self.url = freesound.url
            self.licensename = self._translate_license(freesound.license)
        if configsound is not None:
            self.id = configsound["id"]
            self.username = configsound["username"]
            self.url = configsound["url"]
            self.licensename = configsound["license"]
            self.file = configsound["file"]
        if speechsound is not None:
            self.language = speechsound["language"]
            self.gender = speechsound["gender"]
            self.file = speechsound["file"]

    def _translate_license(self, licenseurl):
        """Translate a license URL into a license  name"""
        if "/licenses/by-nc/" in licenseurl:
            # "Attribution Noncommercial"
            return "by-nc"
        if "/publicdomain/zero/" in licenseurl:
            # "Creative Commons 0"
            return "cc0"
        if "/licenses/by/" in licenseurl:
            # "Attribution"
            return "by"
        return "Unknown"



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['shabda']

package_data = \
{'': ['*'], 'shabda': ['templates/*']}

install_requires = \
['flask[async]',
 'freesound-api',
 'google-cloud-texttospeech',
 'gunicorn',
 'pydub',
 'requests',
 'termcolor',
 'typer[all]']

entry_points = \
{'console_scripts': ['shabda = shabda_cli:cli']}

setup_kwargs = {
    'name': 'shabda',
    'version': '0.1.0',
    'description': 'A semantic audio samples curator for livecoding software',
    'long_description': 'Shabda\n======\n\n![Shabda logo](assets/logo.png)\n\n\nShabda is a tool to fetch random samples from https://freesound.org/ based on given words, for use in impro sessions on instruments such as Tidal Cycles and Estuary.\n\n[Shabda](https://en.wikipedia.org/wiki/Shabda) is the Sanskrit word for "speech sound". In Sanskrit grammar, the term refers to an utterance in the sense of linguistic performance. \n\nInstall\n-------\n\n- Install Python 3: https://www.python.org/\n- Install pip: https://pypi.org/project/pip/\n- Install pipenv: `pip install pipenv`\n- Install dependencies: `pipenv install`\n- Install ffmpeg: https://ffmpeg.org/ (e.g. Debian/Ubuntu: `apt install ffmpeg`)\n\nUse (command line)\n------------------\n\nIn order to download a sample pack, execute in the terminal `pipenv run python shabda_cli.py <definition> --licenses <license_name>`.\n\nAny word can be a pack definition. If you want more than one sample, separate words by a comma: `blue,red`\n\nYou can define how many variations of a sample to assemble by adding a colon and a number.\ne.g. `blue,red:3,yellow:2` will produce one \'blue\' sample, three \'red\' samples and two \'yellow\' sample.\n\nThe optional `--licenses` parameter allows to fetch only samples that have the specified license. Multiple licenses can be allowed by repeating the `--licenses` argument. Possible licenses are `cc0` (Creative Commons Zero), `by` (Creative Commons Attribution), and `by-nc` (Creative Commons Attribution Non-Commercial).\n\nFull example:\n```\npipenv run python shabda_cli.py spaghetti:2,monster:4 --licenses cc0 --licenses by\n```\n\nUse (web application)\n---------------------\n\nLaunch the web application:\n\nIn debug mode:\n```\nFLASK_APP=shabda FLASK_DEBUG=1 pipenv run flask run\n```\nIn production:\n```\npipenv run gunicorn --workers=4 "shabda:create_app()" -b localhost:8000\n```\n\nTest\n----\n\n```\npipenv run pytest\n```\n\nNotes\n-----\n\nWith Estuary, Shabda makes use of this feature: https://github.com/dktr0/estuary/wiki#adding-sound-files-to-estuarywebdirt-on-the-fly\n\nTo do\n-----\n\n- Move all this to github issues or project :)\n- Explain how to launch on codespace / how to make port public\n- Change allowed duration in definition ?\n- List API and view\n- Solidify: Retry, Queue workers, queue status?\n  - https://huey.readthedocs.io/en/latest/index.html\n  - https://github.com/litements/litequeue\n  - (but with those there is no way to know a queue\'s status?)\n- Explain risks about throttling: https://freesound.org/docs/api/overview.html\n- Allow to rename sample ?\n  - Allow emojis in sample name ?\n  - Allow spaces in sample name ? special unicode spaces ? Reverse ?\n- Record your voice\n- Extract percussive ?\n  - https://librosa.org/doc/latest/generated/librosa.effects.hpss.html\n- Use advanced search in a funky way: https://freesound.org/docs/api/analysis_docs.html#analysis-docs\n- Fix refresh token method. Refresh regularly ?\n- Enforce all alpha minuscules and numbers in pack definition\n- Generate speech ;\n    - via external cli ? https://linuxhint.com/command-line-text-speech-apps-linux/\n    - via python lib ? https://pypi.org/project/pyttsx3/\n    - via TTS API? https://rapidapi.com/collection/best-text-to-speech-apis\n- Delete old samples (e.g. 7 days after last usage, note usage date)\n- Autorun pylint\n- Autorun pytest\n- Better cli interface (pack definition, licenses) \n- pip module\n  - Separate web/cli/core\n  - Finish migration from pipenv to Poetry\n  - Store freesound token in a predictable directory\n  - Store samples in a predictable directory (configurable)',
    'author': 'Alexandre G.-Raymond',
    'author_email': 'alex@ndre.gr',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'https://shabda.ndre.gr/',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)



"""Utilities for managing a sample set"""

import os
import json
from glob import glob
from shabda.sound import Sound

FREESOUND = 1
TTS = 2


class SampleSet:
    """A set of sample files"""

    word = None
    master_id = None
    sounds = []
    type = FREESOUND
    samples_path = ""

    def __init__(self, word, soundtype=FREESOUND, samples_path="samples"):
        """Initialize the sample set"""
        self.word = word
        self.type = soundtype
        self.samples_path = samples_path
        directory = self.dir()
        if not os.path.exists(directory):
            os.makedirs(directory)
        else:
            try:
                with open(directory + "/config", encoding="utf-8") as config_file:
                    config = json.load(config_file)
                    self.master_id = config["master"]
                    self.sounds = config["sounds"]
            except IOError:
                self.master_id = None
                self.sounds = []

    def dir(self):
        """Return the directory for this sample set"""
        directory = os.path.join(self.samples_path, self.word)
        if self.type == TTS:
            directory = os.path.join(self.samples_path, "speech_" + directory)
        return directory

    def list(self, max_number=None, licenses=None, gender=None, language=None):
        """List sounds for a sample name"""
        # accept None as a max_number

        sounds = []
        for sound in self.sounds:
            if (
                (licenses is None or sound["license"] in licenses)
                and (gender is None or sound["gender"] == gender)
                and (language is None or sound["language"] == language)
            ):
                sounds.append(Sound(configsound=sound))
        if max_number is not None:
            sounds = sounds[0:max_number]

        return sounds

    def add(self, sound):
        """Add a sound to the sample set"""
        self.sounds.append(
            {
                "id": sound.id,
                "url": sound.url,
                "username": sound.username,
                "license": sound.licensename,
                "file": sound.file,
                "gender": sound.gender,
                "language": sound.language,
            }
        )

    def contains(self, sound_id):
        """Check if a sound id is contained in the sample set"""
        for sound in self.sounds:
            if sound["id"] == sound_id:
                return True
        return False

    def clean(self):
        """Clean the sample set"""
        directory = self.dir()
        if not glob(directory + "/*.wav"):
            os.rmdir(directory)

    def saveconfig(self):
        """Save the master sound ID"""
        directory = self.dir()
        with open(directory + "/config", "w", encoding="utf-8") as config_file:
            json.dump({"master": self.master_id, "sounds": self.sounds}, config_file)



"""Shabda web routes"""

import asyncio
from urllib.parse import urlparse
import json
import os
from zipfile import ZipFile
import tempfile
from flask import (
    Blueprint,
    jsonify,
    send_from_directory,
    request,
    render_template,
    send_file,
    after_this_request,
)
from werkzeug.exceptions import BadRequest, HTTPException
from shabda.dj import Dj


bp = Blueprint("web", __name__, url_prefix="/")

dj = Dj()


@bp.route("/")
def home():
    """Main page"""
    return render_template("home.html")


@bp.route("/pack/<definition>")
async def pack(definition):
    """Retrieve a pack of samples"""
    licenses = request.args.get("licenses")
    if licenses is not None:
        licenses = licenses.split(",")

    tasks = []
    try:
        words = dj.parse_definition(definition)
    except ValueError as ex:
        raise BadRequest(ex) from ex

    for word, number in words.items():
        if number is None:
            number = 1
        tasks.append(fetch_one(word, number, licenses))
    results = await asyncio.gather(*tasks)

    global_status = "empty"
    for status in results:
        if status is True:
            global_status = "ok"

    return jsonify(
        {
            "status": global_status,
            "definition": clean_definition(words),
        }
    )


@bp.route("/<definition>.json")
async def pack_json(definition):
    """Download a reslist definition"""
    complete = request.args.get("complete", False, type=bool)
    licenses = request.args.get("licenses", None)
    if licenses is not None:
        licenses = licenses.split(",")

    await pack(definition)

    url = urlparse(request.base_url)
    base = url.scheme + "://" + url.hostname
    if url.port:
        base += ":" + str(url.port)
    try:
        words = dj.parse_definition(definition)
    except ValueError as ex:
        raise BadRequest(ex) from ex
    reslist = []
    for word, number in words.items():
        samples = dj.list(word, number, licenses=licenses)
        sample_num = 0
        for sound in samples:
            sound_data = {
                "url": sound.file,
                "type": "audio",
                "bank": word,
                "n": sample_num,
            }
            if complete:
                sound_data["licensename"] = sound.licensename
                sound_data["original_url"] = sound.url
                sound_data["author"] = sound.username
            reslist.append(sound_data)
            sample_num += 1

    return jsonify(reslist)


@bp.route("/<definition>.zip")
def pack_zip(definition):
    """Download a zip archive"""
    try:
        words = dj.parse_definition(definition)
    except ValueError as ex:
        raise BadRequest(ex) from ex
    definition = clean_definition(words)
    tmpfile = tempfile.gettempdir() + "/" + definition + ".zip"
    with ZipFile(tmpfile, "w") as zipfile:
        for word, number in words.items():
            samples = dj.list(word, number)
            for sample in samples:
                zipfile.write(sample, sample[len("samples/") :])

    @after_this_request
    def remove_file(response):
        os.remove(tmpfile)
        return response

    return send_file(tmpfile, as_attachment=True)


@bp.route("/speech/<definition>")
async def speech(definition):
    """Download a spoken word"""
    gender = request.args.get("gender", "f")
    language = request.args.get("language", "en-GB")

    definition = definition.replace(" ", "_")
    try:
        words = dj.parse_definition(definition)
    except ValueError as ex:
        raise BadRequest(ex) from ex
    tasks = []
    for word in words:
        tasks.append(speak_one(word, language, gender))
    results = await asyncio.gather(*tasks)
    global_status = "empty"
    for status in results:
        if status is True:
            global_status = "ok"

    return jsonify(
        {
            "status": global_status,
            "definition": clean_definition(words),
        }
    )


@bp.route("/speech/<definition>.json")
async def speech_json(definition):
    """Download a reslist definition"""
    gender = request.args.get("gender", "f")
    language = request.args.get("language", "en-GB")
    definition = definition.replace(" ", "_")

    await speech(definition)

    url = urlparse(request.base_url)
    base = url.scheme + "://" + url.hostname
    if url.port:
        base += ":" + str(url.port)
    try:
        words = dj.parse_definition(definition)
    except ValueError as ex:
        raise BadRequest(ex) from ex
    reslist = []
    for word in words:
        samples = dj.list(word, gender=gender, language=language, soundtype="tts")
        sample_num = 0
        for sound in samples:
            sound_data = {
                "url": sound.file,
                "type": "audio",
                "bank": word,
                "n": sample_num,
            }
            reslist.append(sound_data)
            sample_num += 1

    return jsonify(reslist)


@bp.route("speech/speech_samples/<path:path>")
def serve_sample(path):
    """Serve a sample"""
    return send_from_directory("../speech_samples/", path, as_attachment=False)


@bp.route("/samples/<path:path>")
def serve_speech_sample(path):
    """Serve a sample"""
    return send_from_directory("../samples/", path, as_attachment=False)


@bp.route("/assets/<path:path>")
def static(path):
    """Serve a static asset"""
    return send_from_directory("../assets/", path, as_attachment=False)


@bp.errorhandler(HTTPException)
def handle_exception(exception):
    """Return JSON instead of HTML for HTTP errors."""
    # start with the correct headers and status code from the error
    response = exception.get_response()
    # replace the body with JSON
    response.data = json.dumps(
        {
            "code": exception.code,
            "name": exception.name,
            "description": exception.description,
        }
    )
    response.content_type = "application/json"
    return response


@bp.after_request
def cors_after(response):
    """Add CORS headers to response"""
    response.headers["Access-Control-Allow-Origin"] = "*"
    return response


async def speak_one(word, language, gender):
    """Speak a word"""
    return await dj.speak(word, language, gender)


async def fetch_one(word, number, licenses):
    """Fetch a single sample set"""
    return await dj.fetch(word, number, licenses)


def clean_definition(words):
    """reconstruct the definition without unwanted chars"""
    definition = []
    for word, number in words.items():
        definition.append(word + (":" + str(number) if number else ""))
    return ",".join(definition)
